"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebpackPluginMulticompilerProgress = void 0;
const path_1 = __importDefault(require("path"));
const webpack_1 = require("webpack");
const cli_progress_1 = require("cli-progress");
const chalk_1 = __importDefault(require("chalk"));
const multibar = new cli_progress_1.MultiBar({
    format: `{name} [{bar}] {percentage}% | {msg} ${chalk_1.default.gray('{additional}')}`,
    autopadding: true,
    barsize: 30,
    stopOnComplete: true,
});
const bars = new Map();
function parseAdditionalInfo(type, str) {
    if (type === 'building') {
        const activeModule = str[1];
        if (activeModule) {
            const importWithLoaders = activeModule.split(/(!|\|)/).filter(Boolean);
            const last = importWithLoaders[importWithLoaders.length - 1];
            return path_1.default.isAbsolute(last) ? path_1.default.relative(process.cwd(), last) : last;
        }
        return '';
    }
    return str.join(' ');
}
function progressHandler(compiler, name, percentage, msg, ...args) {
    const padEndName = Math.max(...Array.from(bars.keys()).map((name) => name.length));
    const padStartMsg = Math.max(...Array.from(bars.values()).map((bar) => bar ? bar.payload.msg.trim().length : 0));
    const payload = {
        name: name.padEnd(padEndName),
        msg: msg.padStart(padStartMsg),
        additional: parseAdditionalInfo(msg, args),
    };
    if (bars.has(name)) {
        const bar = bars.get(name);
        bar?.update(percentage, payload);
    }
    else {
        const bar = multibar.create(1, percentage, payload);
        bars.set(name, bar);
    }
}
class WebpackPluginMulticompilerProgress {
    constructor(...params) {
        if (WebpackPluginMulticompilerProgress.instance) {
            return WebpackPluginMulticompilerProgress.instance;
        }
        this.params = params;
        WebpackPluginMulticompilerProgress.instance = this;
    }
    apply(compiler) {
        const opts = this.params[0];
        const plugin = new webpack_1.ProgressPlugin({
            ...opts,
            activeModules: true,
            handler(...args) {
                progressHandler(compiler, String(compiler.options.name ?? compiler.options.target ?? ''), ...args);
            },
        });
        plugin.apply(compiler);
    }
}
exports.WebpackPluginMulticompilerProgress = WebpackPluginMulticompilerProgress;
//# sourceMappingURL=webpack-plugin-multicompiler-progress.js.map